"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JoinQueryBuilder = void 0;
var DomainError_1 = require("tstl/exception/DomainError");
var OutOfRange_1 = require("tstl/exception/OutOfRange");
var ReflectAdaptor_1 = require("../decorators/base/ReflectAdaptor");
var RelationshipVariable_1 = require("../decorators/base/RelationshipVariable");
var ITableInfo_1 = require("../functional/internal/ITableInfo");
var functional_1 = require("../functional");
/**
 * DB level join query builder.
 *
 * `JoinQueryBuilder` is a helper class who can perform the DB level join very
 * conveniently and safely.
 *
 * When you are building up an `JoinQueryBuilder` instance from a specific ORM class,
 * the `JoinQueryBuilder` would analyze relationships of the target ORM class in the
 * compilation level. Therefore, if you take any mistake, it would be detected in the
 * IDE directly. Also, when you're finding neighbor entities to join, `JoinQueryBuilder`
 * will hint you through the auto-completion.
 *
 * ![Hints by the Auto Completion](https://raw.githubusercontent.com/samchon/safe-typeorm/master/assets/demonstrations/safe-query-builder.gif)
 *
 * Therefore, you don't need to write any raw SQL join query of the TypeORM more. You
 * don't need to suffer from the run-time join SQL error of the TypeORM more. Just
 * replace the join queries to use this `JoinQueryBuilder` instance and feel free from
 * those pains.
 *
 * Also, if you want to perform the DB level join query not to specifying (a) specific
 * record(s) (through where condition from the neighborhood entity) but to combine data,
 * I recommend you to use {@link AppJoinBuilder} or {@link JsonSelectBuilder} rather
 * than this `JoinQueryBuilder`. It's the reason why the application level join
 * consumes much fewer resource and elapses much shorter time.
 *
 * Type         | DB Join   | App Join
 * :------------|----------:|----------:
 * Records      | 2,258,000 | 165
 * Elapsed Time | 8.07508   | 0.00262
 *
 * @template Mine Target entity to perform the DB join
 * @template Query Target entity of `TypeORM.SelectQueryBuilder`
 * @reference [stackoverflow/join-queries-vs-multiple-queries](https://stackoverflow.com/questions/1067016/join-queries-vs-multiple-queries)
 * @author Jeongho Nam - https://github.com/samchon
 */
var JoinQueryBuilder = /** @class */ (function () {
    /**
     * @hidden
     */
    function JoinQueryBuilder(stmt, mine, alias) {
        this.stmt_ = stmt;
        this.mine_ = mine;
        this.alias_ = alias === undefined ? mine.name : alias;
        this.joined_ = new Map();
    }
    /* -----------------------------------------------------------
        CONNSTRUCTOR
    ----------------------------------------------------------- */
    /**
     * @internal
     */
    JoinQueryBuilder.create = function (stmt, mine, alias) {
        return new JoinQueryBuilder({
            query: stmt,
            groupped: false,
            ordered: false,
            selected: false,
        }, mine, alias);
    };
    JoinQueryBuilder.prototype._Take_join = function (method, field, alias, closure, joiner) {
        // PREPARE ASSET
        var asset = prepare_asset(this.mine_, field, alias);
        // CHECK OLDBIE
        var oldbie = this.joined_.get(field);
        if (oldbie !== undefined) {
            if (method !== oldbie.method)
                throw new DomainError_1.DomainError("Error on safe.JoinQueryBuilder.".concat(method, "(): ").concat(this.mine_.name, ".").concat(field, " already has been joined by ").concat(oldbie.method, "()."));
            else if (asset.alias !== oldbie.alias)
                throw new DomainError_1.DomainError("Error on safe.JoinQueryBuilder.".concat(method, "(): ").concat(this.mine_.name, ".").concat(field, " already has been joined with another alias \"").concat(oldbie.alias, "\"."));
            return oldbie.builder;
        }
        // DO JOIN
        joiner(asset);
        // BUILDER
        var builder = new JoinQueryBuilder(this.stmt_, asset.metadata.target(), asset.alias);
        this.joined_.set(field, { method: method, alias: asset.alias, builder: builder });
        if (closure)
            closure(builder);
        return builder;
    };
    /* -----------------------------------------------------------
        ACCESSORS
    ----------------------------------------------------------- */
    JoinQueryBuilder.prototype.has = function (field) {
        return this.joined_.has(field);
    };
    JoinQueryBuilder.prototype.get = function (field) {
        var found = this.joined_.get(field);
        if (found === undefined)
            throw new OutOfRange_1.OutOfRange("Error on safe.JoinQueryBuilder.get(): unable to find the matched key - \"".concat(field, "\"."));
        return found.builder;
    };
    JoinQueryBuilder.prototype.size = function () {
        return this.joined_.size;
    };
    JoinQueryBuilder.prototype.empty = function () {
        return this.size() === 0;
    };
    JoinQueryBuilder.prototype.statement = function () {
        return this.stmt_.query;
    };
    /* -----------------------------------------------------------
        STATEMENTS
    ----------------------------------------------------------- */
    JoinQueryBuilder.prototype._Decompose_field_like = function (fieldLike) {
        var _this = this;
        var escape = function (field) {
            var _a;
            return ((_a = ReflectAdaptor_1.ReflectAdaptor.get(_this.mine_.prototype, field)) === null || _a === void 0 ? void 0 : _a.foreign_key_field) || field;
        };
        var symbol = function (column) { return "".concat(_this.alias_, ".").concat(column); };
        if (typeof fieldLike === "string") {
            var column = escape(fieldLike);
            return { column: column, symbol: symbol(column) };
        }
        else {
            var column = escape(fieldLike[0]);
            var closure = fieldLike[1];
            return {
                column: column,
                symbol: closure(symbol(column)),
            };
        }
    };
    JoinQueryBuilder.prototype.addSelect = function (fieldLike, alias) {
        var _a = this._Decompose_field_like(fieldLike), column = _a.column, symbol = _a.symbol;
        alias || (alias = column);
        if (this.stmt_.selected === false) {
            this.stmt_.query.select(symbol, alias);
            this.stmt_.selected = true;
        }
        else {
            this.stmt_.query.addSelect(symbol, alias || column);
        }
        return this;
    };
    JoinQueryBuilder.prototype.addOrderBy = function (fieldLike, order, nulls) {
        var symbol = this._Decompose_field_like(fieldLike).symbol;
        if (this.stmt_.ordered === false) {
            this.stmt_.query.orderBy(symbol, order, nulls);
            this.stmt_.ordered = true;
        }
        else {
            this.stmt_.query.addOrderBy(symbol, order, nulls);
        }
        return this;
    };
    JoinQueryBuilder.prototype.addGroupBy = function (fieldLike) {
        var symbol = this._Decompose_field_like(fieldLike).symbol;
        if (this.stmt_.groupped === false) {
            this.stmt_.query.groupBy(symbol);
            this.stmt_.groupped = true;
        }
        else {
            this.stmt_.query.addGroupBy(symbol);
        }
        return this;
    };
    JoinQueryBuilder.prototype.andWhere = function (fieldLike) {
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        this._Where.apply(this, __spreadArray([function (stmt) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                return stmt.andWhere.apply(stmt, __spreadArray([], __read(args), false));
            },
            fieldLike], __read(rest), false));
        return this;
    };
    JoinQueryBuilder.prototype.orWhere = function (fieldLike) {
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        this._Where.apply(this, __spreadArray([function (stmt) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                return stmt.orWhere.apply(stmt, __spreadArray([], __read(args), false));
            },
            fieldLike], __read(rest), false));
        return this;
    };
    JoinQueryBuilder.prototype._Where = function (condition, fieldLike) {
        var rest = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            rest[_i - 2] = arguments[_i];
        }
        var parameters = [
            this.mine_,
            typeof fieldLike === "string"
                ? "".concat(this.alias_, ".").concat(fieldLike)
                : ["".concat(this.alias_, ".").concat(fieldLike[0]), fieldLike[1]],
        ];
        parameters.push.apply(parameters, __spreadArray([], __read(rest), false));
        var args = functional_1.getWhereArguments.apply(void 0, __spreadArray([], __read(parameters), false));
        condition.apply(void 0, __spreadArray([this.stmt_.query], __read(args), false));
    };
    JoinQueryBuilder.prototype.getColumn = function (field) {
        var _a;
        var column = ((_a = ReflectAdaptor_1.ReflectAdaptor.get(this.mine_.prototype, field)) === null || _a === void 0 ? void 0 : _a.foreign_key_field) || field;
        return "".concat(this.alias_, ".").concat(column);
    };
    JoinQueryBuilder.prototype.innerJoin = function (field, alias, closure) {
        return this._Join_atomic.apply(this, __spreadArray(["innerJoin",
            field], __read(get_parametric_tuple(alias, closure)), false));
    };
    JoinQueryBuilder.prototype.leftJoin = function (field, alias, closure) {
        return this._Join_atomic.apply(this, __spreadArray(["leftJoin",
            field], __read(get_parametric_tuple(alias, closure)), false));
    };
    JoinQueryBuilder.prototype.innerJoinAndSelect = function (field, alias, closure) {
        return this._Join_and_select.apply(this, __spreadArray(["innerJoinAndSelect",
            field], __read(get_parametric_tuple(alias, closure)), false));
    };
    JoinQueryBuilder.prototype.leftJoinAndSelect = function (field, alias, closure) {
        return this._Join_and_select.apply(this, __spreadArray(["leftJoinAndSelect",
            field], __read(get_parametric_tuple(alias, closure)), false));
    };
    JoinQueryBuilder.prototype._Join_atomic = function (method, field, alias, closure) {
        var _this = this;
        return this._Take_join(method, field, alias, closure, function (asset) {
            // LIST UP EACH FIELDS
            var _a = __read((function () {
                if (asset.belongs === true)
                    return [
                        asset.metadata.foreign_key_field,
                        get_primary_column(asset.metadata.target()),
                    ];
                var inverseMetadata = ReflectAdaptor_1.ReflectAdaptor.get(asset.metadata.target().prototype, asset.metadata.inverse);
                return [
                    get_primary_column(_this.mine_),
                    inverseMetadata.foreign_key_field,
                ];
            })(), 2), myField = _a[0], targetField = _a[1];
            // DO JOIN
            var condition = "".concat(_this.alias_, ".").concat(myField, " = ").concat(asset.alias, ".").concat(targetField);
            _this.stmt_.query[method](asset.metadata.target(), asset.alias, condition);
        });
    };
    JoinQueryBuilder.prototype._Join_and_select = function (method, field, alias, closure) {
        var _this = this;
        return this._Take_join(method, field, alias, closure, function (asset) {
            var index = RelationshipVariable_1.RelationshipVariable.getter(asset.belongs ? "belongs" : "has", field);
            _this.stmt_.query[method]("".concat(_this.alias_, ".").concat(index), asset.alias);
        });
    };
    return JoinQueryBuilder;
}());
exports.JoinQueryBuilder = JoinQueryBuilder;
function prepare_asset(mine, field, alias) {
    var metadata = ReflectAdaptor_1.ReflectAdaptor.get(mine.prototype, field);
    var belongs = metadata.type.indexOf("Belongs.") === 0;
    // DETERMINE THE ALIAS
    if (alias === undefined)
        alias = metadata.target().name;
    // RETURNS
    return {
        belongs: belongs,
        metadata: metadata,
        alias: alias,
    };
}
function get_parametric_tuple(x, y) {
    return typeof x === "string" ? [x, y] : [undefined, x];
}
function get_primary_column(creator) {
    return ITableInfo_1.ITableInfo.get(creator).primaryColumn;
}
//# sourceMappingURL=JoinQueryBuilder.js.map