import * as orm from "typeorm";
import { CapsuleNullable } from "../../typings/CapsuleNullable";
import { Creator } from "../../typings/Creator";
import { PrimaryColumnType } from "../../typings/PrimaryColumnType";
import { StringColumnType } from "../../typings/StringColumnType";
import { BelongsAccessorBase } from "../base/BelongsAccessorBase";
import { HasOneToOne } from "./HasOneToOne";
/**
 * Type of a variable using the `Belongs.OneToOne` decorator.
 *
 * @template Target Type of the target model class who has this model class as 1: 1
 * @template Type Type of a type for a primary key column of the *Target* model class
 * @template Options Type of the surplus options defining the foreign referencing column
 */
export declare type BelongsOneToOne<Target extends object, Type extends PrimaryColumnType, Options extends Partial<BelongsOneToOne.IOptions<Type>> = {}> = BelongsOneToOne.Accessor<Target, Type, Options>;
/**
 * Decorator function for the "1: 1 belongs" relationship.
 *
 * @template Target Type of the target model class who has this model class as 1: 1
 * @template Type Type of a type for a primary key column of the *Target* model class
 * @template Options Type of the surplus options describing the foreign referencing column
 * @param targetGen A closure function returning the *Target* model class, who has this model
 *                  class as 1: 1. In other words, this model class belongs to the *Target*
 *                  model class as 1: 1
 * @param type Type of a primary key column of the *Target* model class
 * @param myField Column name of this table who references the *Target* table as a foreign key
 * @param options Surplus options decribing the foreign referencing column *myField*
 * @return The *PropertyDecorator* function
 */
export declare function BelongsOneToOne<Target extends object, Type extends PrimaryColumnType, Options extends Partial<BelongsOneToOne.IOptions<Type>>>(targetGen: Creator.Generator<Target>, type: Type, myField: string, options?: Options): PropertyDecorator;
/**
 * Decorator funtion for the "1: 1 belongs" relationship.
 *
 * @template Mine Type of this model class who is using it, the `Belongs.OneToOne` decorator
 * @template Target Type of the target model class who has this model class *Mine* as 1: 1
 * @template Type Type of the primary key of the *Target* model class
 * @template Options Type of the surplus options describing the foreign referencing column
 * @param targetGen A closure function returning the *Target* model class, who has this model
 *                  class *Mine* as 1: 1. In other words, this model class *Mine* belongs to
 *                  the *Target* model class as 1: 1
 * @param inverse A closure function returning the {@link Has.OneToOne} typed member variable,
 *                who is pointing this model class *Mine*, from the *Target* model class
 * @param type Type of a primary key column of the *Target* model class
 * @param myField Column name of *Mine* who references the *Target* table as a foreign key
 * @param options Surplus options decribing the foreign referencing column *myField*
 * @return The *PropertyDecorator* function
 */
export declare function BelongsOneToOne<Mine extends object, Target extends object, Type extends PrimaryColumnType, Options extends Partial<BelongsOneToOne.IOptions<Type>>>(targetGen: Creator.Generator<Target>, inverse: (target: Target) => HasOneToOne<Mine>, type: Type, myField: string, options?: Options): PropertyDecorator;
export declare namespace BelongsOneToOne {
    interface IOptions<Type extends PrimaryColumnType> extends Required<Omit<orm.RelationOptions, "primary" | "eager" | "lazy">> {
        index: boolean;
        primary: boolean;
        unique: boolean;
        unsigned: Type extends StringColumnType ? never : boolean;
        length: number;
    }
    class Accessor<Target extends object, Type extends PrimaryColumnType, Options extends Partial<IOptions<Type>>> extends BelongsAccessorBase<Target, Type, Options> {
        private readonly metadata_;
        private readonly mine_;
        private singleton_;
        private constructor();
        /**
         *
         */
        get id(): CapsuleNullable<PrimaryColumnType.ValueType<Type>, Options>;
        set id(value: CapsuleNullable<PrimaryColumnType.ValueType<Type>, Options>);
        statement(): orm.QueryBuilder<Target>;
        set(obj: CapsuleNullable<Target, Options>): Promise<void>;
        reload(): Promise<CapsuleNullable<Target, Options>>;
        get(): Promise<CapsuleNullable<Target, Options>>;
        private _Get;
        private get_null_error_message;
    }
}
